#
# DATABRICKS CONFIDENTIAL & PROPRIETARY
# __________________
#
# Copyright 2020-present Databricks, Inc.
# All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of Databricks, Inc.
# and its suppliers, if any.  The intellectual and technical concepts contained herein are
# proprietary to Databricks, Inc. and its suppliers and may be covered by U.S. and foreign Patents,
# patents in process, and are protected by trade secret and/or copyright law. Dissemination, use,
# or reproduction of this information is strictly forbidden unless prior written permission is
# obtained from Databricks, Inc.
#
# If you view or obtain a copy of this information and believe Databricks, Inc. may not have
# intended it to be made available, please promptly report it to Databricks Legal Department
# @ legal@databricks.com.
#

from pyspark.databricks.testing.utils import UsageLoggingTestCase


class PandasOnSparkUsageLoggingTest(UsageLoggingTestCase):
    """
    Test suite for pandas-on-Spark usage logging.
    """

    def test_pandas_on_spark_usage_logging(self):
        def function_with_logging():
            import pyspark.pandas as ps
            from pyspark.pandas.exceptions import PandasNotImplementedError

            psdf = ps.DataFrame({"a": [1, 2, 3], "b": ["x", "y", "z"]})

            self.assertEquals(list(psdf.columns), ["a", "b"])

            self.assertRaises(AttributeError, lambda: psdf.c)
            self.assertRaises(PandasNotImplementedError, lambda: psdf.to_pickle())

        usage_records = list(self.track_usage(function_with_logging))

        self.assertEquals(
            usage_records[0].metric(),
            self.metric_definitions().EVENT_PANDAS_ON_SPARK_IMPORTED().name(),
        )
        self.assertEquals(
            usage_records[1].metric(),
            self.metric_definitions().EVENT_INITIAL_CONFIG_LOG().name(),
        )
        self.assertTrue(
            all(
                record.metric()
                == self.metric_definitions().EVENT_PANDAS_ON_SPARK_FUNCTION_CALLED().name()
                for record in usage_records[2:]
            )
        )

        self.assertTrue(
            self.get_tag_value(
                usage_records[2], self.tag_definitions().TAG_PANDAS_ON_SPARK_FUNCTION()
            ).startswith("__init__")
        )
        self.assert_tag(
            usage_records[2],
            self.tag_definitions().TAG_STATUS(),
            "success",
        )
        self.assert_tag(
            usage_records[3],
            self.tag_definitions().TAG_PANDAS_ON_SPARK_PROPERTY(),
            "columns",
        )
        self.assert_tag(
            usage_records[3],
            self.tag_definitions().TAG_STATUS(),
            "success",
        )

        failure_records = [
            record
            for record in usage_records[2:]
            if self.get_tag_value(record, self.tag_definitions().TAG_STATUS()) == "failure"
        ]
        self.assertEquals(len(failure_records), 1)
        self.assertTrue(
            self.get_tag_value(
                failure_records[0], self.tag_definitions().TAG_PANDAS_ON_SPARK_FUNCTION()
            ).startswith("__getattr__")
        )

        missing_records = [
            record
            for record in usage_records[2:]
            if self.get_tag_value(record, self.tag_definitions().TAG_STATUS()) == "missing"
        ]
        self.assertEquals(len(missing_records), 1)
        self.assertTrue(
            self.get_tag_value(
                missing_records[0], self.tag_definitions().TAG_PANDAS_ON_SPARK_FUNCTION()
            ).startswith("to_pickle")
        )


if __name__ == "__main__":
    import unittest
    from pyspark.databricks.pandas.tests.test_usage_logging import *  # noqa: F401,F403

    try:
        import xmlrunner

        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2),
            verbosity=2,
        )
    except ImportError:
        unittest.main(verbosity=2)
