#
# DATABRICKS CONFIDENTIAL & PROPRIETARY
# __________________
#
# Copyright 2020-present Databricks, Inc.
# All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of Databricks, Inc.
# and its suppliers, if any.  The intellectual and technical concepts contained herein are
# proprietary to Databricks, Inc. and its suppliers and may be covered by U.S. and foreign Patents,
# patents in process, and are protected by trade secret and/or copyright law. Dissemination, use,
# or reproduction of this information is strictly forbidden unless prior written permission is
# obtained from Databricks, Inc.
#
# If you view or obtain a copy of this information and believe Databricks, Inc. may not have
# intended it to be made available, please promptly report it to Databricks Legal Department
# @ legal@databricks.com.
#

import unittest

from pyspark.sql.tests.pandas.test_pandas_udf_scalar import ScalarPandasUDFTestsBase
from pyspark.testing.sqlutils import (
    ReusedSQLTestCase,
)

import pandas as pd
from pyspark.sql import Row
from pyspark.sql.functions import struct, pandas_udf


class SafeSparkPandasScalarUDFTests(ScalarPandasUDFTestsBase, ReusedSQLTestCase):
    @classmethod
    def setUpClass(cls):
        super(ScalarPandasUDFTestsBase, cls).setUpClass()
        # Ensure the isolated udf is enabled
        assert cls.spark.conf.get("spark.databricks.pyspark.isolatedUDF.enabled") == "true"
        cls.spark.conf.set("spark.databricks.pyspark.isolatedPandasUDF.enabled", "true")
        assert cls.spark.conf.get("spark.databricks.pyspark.isolatedPandasUDF.enabled") == "true"

    @classmethod
    def tearDownClass(cls):
        super(ScalarPandasUDFTestsBase, cls).tearDownClass()

    @unittest.skip("Too many UDF calls, too expensive to run.")
    def test_mixed_udf(self):
        super(SafeSparkPandasScalarUDFTests, self).test_mixed_udf()

    @unittest.skip("Too many UDF calls, too expensive to run.")
    def test_mixed_udf_and_sql(self):
        super(SafeSparkPandasScalarUDFTests, self).test_mixed_udf_and_sql()

    @unittest.skip("SAS-891: disable unsupported types as pyspark.")
    def test_vectorized_udf_nested_struct(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_nested_struct()

    @unittest.skip("SAS-891: disable unsupported types as pyspark.")
    def test_vectorized_udf_unsupported_types(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_unsupported_types()

    @unittest.skip("SAS-891: disable unsupported types as pyspark.")
    def test_vectorized_udf_wrong_return_type(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_wrong_return_type()

    @unittest.skip("SAS-892: finally block in scalar iterator udf not reached.")
    def test_scalar_iter_udf_close(self):
        super(SafeSparkPandasScalarUDFTests, self).test_scalar_iter_udf_close()

    @unittest.skip("SAS-892: finally block in scalar iterator udf not reached.")
    def test_scalar_iter_udf_close_early(self):
        super(SafeSparkPandasScalarUDFTests, self).test_scalar_iter_udf_close_early()

    @unittest.skip("SAS-850: Support TaskContext in safe spark.")
    def test_scalar_iter_udf_init(self):
        super(SafeSparkPandasScalarUDFTests, self).test_scalar_iter_udf_init()

    @unittest.skip("SAS-893: Respect spark.sql.execution.arrow.maxRecordsPerBatch in safe spark.")
    def test_vectorized_udf_check_config(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_check_config()

    @unittest.skip(
        "SAS-889: unify error message when scalar UDF produces incorrect number of rows in output."
    )
    def test_vectorized_udf_invalid_length(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_invalid_length()

    @unittest.skip("SAS-1061: clashes with asyncio unblocking of udf server")
    def test_vectorized_udf_chained_struct_type(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_chained_struct_type()

    @unittest.skip("SAS-775: return type coercion is not supported.")
    def test_vectorized_udf_struct_with_empty_partition(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_struct_with_empty_partition()

    @unittest.skip("SAS-870: timestamp after arrow => pandas conversion always has a timezone.")
    def test_vectorized_udf_timestamps(self):
        super(SafeSparkPandasScalarUDFTests, self).test_vectorized_udf_timestamps()

    @unittest.skip("SAS-870: timestamp after arrow => pandas conversion always has a timezone.")
    def test_vectorized_udf_timestamps_respect_session_timezone(self):
        super(
            SafeSparkPandasScalarUDFTests, self
        ).test_vectorized_udf_timestamps_respect_session_timezone()

    def test_struct_input(self):
        @pandas_udf("string")
        def struct_input_udf(input):
            assert isinstance(input, pd.DataFrame)
            return input["x"]

        self.assertEquals(
            self.spark.createDataFrame([Row(x="hello")])
            .select(struct_input_udf(struct("x")))
            .collect()[0][0],
            "hello",
        )


if __name__ == "__main__":
    from pyspark.databricks.sql.tests.safespark.test_safespark_pandas_scalar_udf import *  # noqa: E501,F401,F403

    try:
        import xmlrunner

        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2),
            verbosity=2,
        )
    except ImportError:
        unittest.main(verbosity=2)
