#
# DATABRICKS CONFIDENTIAL & PROPRIETARY
# __________________
#
# Copyright 2020-present Databricks, Inc.
# All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of Databricks, Inc.
# and its suppliers, if any.  The intellectual and technical concepts contained herein are
# proprietary to Databricks, Inc. and its suppliers and may be covered by U.S. and foreign Patents,
# patents in process, and are protected by trade secret and/or copyright law. Dissemination, use,
# or reproduction of this information is strictly forbidden unless prior written permission is
# obtained from Databricks, Inc.
#
# If you view or obtain a copy of this information and believe Databricks, Inc. may not have
# intended it to be made available, please promptly report it to Databricks Legal Department
# @ legal@databricks.com.
#

import unittest

from pyspark.sql.functions import udf
from pyspark.sql.tests.test_udf import BaseUDFTestsMixin
from pyspark.testing.sqlutils import (
    have_pandas,
    have_pyarrow,
    pandas_requirement_message,
    pyarrow_requirement_message,
    ReusedSQLTestCase,
)


@unittest.skipIf(
    not have_pandas or not have_pyarrow, pandas_requirement_message or pyarrow_requirement_message
)
class PythonUDFArrowTests(BaseUDFTestsMixin, ReusedSQLTestCase):
    @classmethod
    def setUpClass(cls):
        super(PythonUDFArrowTests, cls).setUpClass()
        cls.spark.conf.set("spark.databricks.execution.pythonUDF.arrow.enabled", "true")

    @unittest.skip("Unrelated test, and it fails when it runs duplicatedly.")
    def test_broadcast_in_udf(self):
        super(PythonUDFArrowTests, self).test_broadcast_in_udf()

    @unittest.skip("Unrelated test, and it fails when it runs duplicatedly.")
    def test_register_java_function(self):
        super(PythonUDFArrowTests, self).test_register_java_function()

    @unittest.skip("Unrelated test, and it fails when it runs duplicatedly.")
    def test_register_java_udaf(self):
        super(PythonUDFArrowTests, self).test_register_java_udaf()

    @unittest.skip("Struct input types are not supported with Arrow optimization")
    def test_udf_input_serialization_valuecompare_disabled(self):
        super(PythonUDFArrowTests, self).test_udf_input_serialization_valuecompare_disabled()

    def test_nested_input_error(self):
        with self.assertRaisesRegexp(
            Exception, "NotImplementedError: Struct input type are not supported"
        ):
            self.spark.range(1).selectExpr("struct(1, 2) as struct").select(
                udf(lambda x: x)("struct")
            ).collect()

    def test_complex_input_types(self):
        # TODO: add map type tests back when PyArrow is upgraded.
        row = (
            self.spark.range(1)
            .selectExpr(
                "array(1, 2, 3) as array",
                # "map('a', 'b') as map"
            )
            .select(
                udf(lambda x: str(x))("array"),
                # udf(lambda x: str(x))("map"),
            )
            .first()
        )

        # The input is NumPy array when the optimization is on.
        self.assertEquals(row[0], "[1 2 3]")
        # self.assertEquals(row[1], "{'a': 'b'}")


if __name__ == "__main__":
    from pyspark.databricks.sql.tests.test_arrow_python_udf import *  # noqa: F401,F403

    try:
        import xmlrunner

        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2),
            verbosity=2,
        )
    except ImportError:
        unittest.main(verbosity=2)
