#
# DATABRICKS CONFIDENTIAL & PROPRIETARY
# __________________
#
# Copyright 2020-present Databricks, Inc.
# All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property of Databricks, Inc.
# and its suppliers, if any.  The intellectual and technical concepts contained herein are
# proprietary to Databricks, Inc. and its suppliers and may be covered by U.S. and foreign Patents,
# patents in process, and are protected by trade secret and/or copyright law. Dissemination, use,
# or reproduction of this information is strictly forbidden unless prior written permission is
# obtained from Databricks, Inc.
#
# If you view or obtain a copy of this information and believe Databricks, Inc. may not have
# intended it to be made available, please promptly report it to Databricks Legal Department
# @ legal@databricks.com.
#

import re
import unittest
from pyspark.databricks.sql.functions import unwrap_udt, approx_top_k
from pyspark.sql import Row
from pyspark.sql.functions import col, lit
from pyspark.testing.sqlutils import ReusedSQLTestCase


class FunctionsTests(ReusedSQLTestCase):
    def test_unwrap_udt(self):
        from pyspark.ml.linalg import Vectors

        df = self.spark.createDataFrame(
            [(Vectors.dense(1.0, 2.0, 3.0),), (Vectors.sparse(3, {1: 1.0, 2: 5.5}),)], ["vec"]
        )
        results = df.select(unwrap_udt("vec").alias("v2")).collect()
        unwrapped_vec = Row("type", "size", "indices", "values")
        expected = [
            Row(v2=unwrapped_vec(1, None, None, [1.0, 2.0, 3.0])),
            Row(v2=unwrapped_vec(0, 3, [1, 2], [1.0, 5.5])),
        ]
        self.assertEquals(results, expected)

    def test_sampleby(self):
        from pyspark.sql.functions import col

        dataset = self.spark.range(0, 100).select((col("id") % 3).alias("key"))
        sampled = dataset.sampleBy("key", fractions={0: 0.1, 1: 0.2}, seed=0)
        self.assertTrue(sampled.count() == 9)

    def test_approx_top_k(self):
        actual = [
            re.findall("(approx_top_k\\(.*\\))", str(x))[0]
            for x in [
                approx_top_k(col("baz")),
                approx_top_k(col("foo"), lit(6)),
                approx_top_k(col("bar"), 50),
                approx_top_k(col("bar"), 50, 100),
                approx_top_k(col("foo"), 5, lit(5) + lit(5)),
            ]
        ]
        expected = [
            "approx_top_k(baz, 5, 10000)",
            "approx_top_k(foo, 6, 10000)",
            "approx_top_k(bar, 50, 10000)",
            "approx_top_k(bar, 50, 100)",
            "approx_top_k(foo, 5, (5 + 5))",
        ]

        self.assertListEqual(actual, expected)


if __name__ == "__main__":
    try:
        import xmlrunner

        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2),
            verbosity=2,
        )
    except ImportError:
        unittest.main(verbosity=2)
