"""
DATABRICKS CONFIDENTIAL & PROPRIETARY
__________________

Copyright 2019 Databricks, Inc.
All Rights Reserved.

NOTICE:  All information contained herein is, and remains the property of Databricks, Inc.
and its suppliers, if any.  The intellectual and technical concepts contained herein are
proprietary to Databricks, Inc. and its suppliers and may be covered by U.S. and foreign Patents,
patents in process, and are protected by trade secret and/or copyright law. Dissemination, use,
or reproduction of this information is strictly forbidden unless prior written permission is
obtained from Databricks, Inc.

If you view or obtain a copy of this information and believe Databricks, Inc. may not have
intended it to be made available, please promptly report it to Databricks Legal Department
@ legal@databricks.com.
"""

import unittest

from pyspark import SparkConf, SparkContext
from pyspark.sql import SparkSession


class DBConnectBroadcastTest(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        conf = (
            SparkConf()
            .set("spark.databricks.service.client.enabled", "true")
            .set("spark.databricks.service.address", "localhost")
            .set("spark.databricks.pyspark.dbconnect.enableProcessIsolation", "true")
        )
        self.sc = SparkContext("local-cluster[1,4,1024]", self.__class__.__name__, conf=conf)
        self.sc.setLocalProperty("spark.databricks.non.uc.token", "non-uc-token")
        # Has to initialize the spark session otherwise dbconnect wouldn't be triggered
        SparkSession.builder.getOrCreate()

    def tearDown(self):
        self.sc.stop()

    def test_broadcast(self):
        lst = [1, 2, 3, 4, 5]
        b = self.sc.broadcast(lst)
        self.assertTrue(b.value == [1, 2, 3, 4, 5])
        result = self.sc.parallelize([0, 0]).flatMap(lambda x: b.value).collect()
        self.assertTrue(result == (lst + lst))


if __name__ == "__main__":
    from pyspark.tests.test_broadcast_dbconnect import *  # noqa: F401

    try:
        import xmlrunner

        testRunner = xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2)
    except ImportError:
        testRunner = None
    unittest.main(testRunner=testRunner, verbosity=2)
