"""
DATABRICKS CONFIDENTIAL & PROPRIETARY
__________________

Copyright 2021 Databricks, Inc.
All Rights Reserved.

NOTICE:  All information contained herein is, and remains the property of Databricks, Inc.
and its suppliers, if any.  The intellectual and technical concepts contained herein are
proprietary to Databricks, Inc. and its suppliers and may be covered by U.S. and foreign Patents,
patents in process, and are protected by trade secret and/or copyright law. Dissemination, use,
or reproduction of this information is strictly forbidden unless prior written permission is
obtained from Databricks, Inc.

If you view or obtain a copy of this information and believe Databricks, Inc. may not have
intended it to be made available, please promptly report it to Databricks Legal Department
@ legal@databricks.com.
"""

import unittest
import base64

from pyspark import SparkContext, SparkConf
from pyspark.sql import SparkSession


# [SC-71696]
# Test the Spark Conf Redaction util for PySpark
class SparkConfRedactTests(unittest.TestCase):
    _testKey1 = "spark.test.key.1"
    _testKey2 = "spark.test.key.2"
    _testKey3 = "spark.test.key.3"
    _testVal = "value"
    # Configure the testKey1 as a sensitive conf key
    _testKey1Base64 = base64.b64encode(_testKey1.encode("UTF-8")).decode("UTF-8")
    _testKey3Base64 = base64.b64encode(_testKey3.encode("UTF-8")).decode("UTF-8")

    def setUpSparkContext(self, redact_enabled=True):
        conf = (
            SparkConf()
            .set(
                "spark.databricks.pyspark.py4jConfRedaction.enabled",
                "true" if redact_enabled else "false",
            )
            .set(
                "spark.databricks.secret.sparkConf.keys.toRedact",
                ",".join([self._testKey1Base64, self._testKey3Base64]),
            )
            .set(self._testKey1, self._testVal)
            .set(self._testKey2, self._testVal)
            .set("spark.databricks.pyspark.dbconnect.enableProcessIsolation", "true")
        )
        self.sc = SparkContext(conf=conf)

    def tearDown(self):
        if getattr(self, "sc", None) is not None:
            self.sc.stop()
            self.sc = None

    def test_spark_conf_get(self):
        self.setUpSparkContext()
        self.assertEqual(self.sc.getConf().get(self._testKey1), "[REDACTED]")
        self.assertEqual(self.sc.getConf().get(self._testKey2), self._testVal)
        self.assertEqual(self.sc.getConf().get(self._testKey1, "default"), "[REDACTED]")

    def test_spark_conf_getAll(self):
        self.setUpSparkContext()
        allConfKv = self.sc.getConf().getAll()
        self.assertTrue((self._testKey1, "[REDACTED]") in allConfKv)
        self.assertTrue((self._testKey2, self._testVal) in allConfKv)
        self.assertTrue((self._testKey1, self._testVal) not in allConfKv)

    def test_spark_session_conf_get(self):
        self.setUpSparkContext()
        spark = SparkSession(self.sc)
        self.assertEqual(spark.conf.get(self._testKey1), "[REDACTED]")
        self.assertEqual(spark.conf.get(self._testKey2), self._testVal)
        self.assertEqual(spark.conf.get(self._testKey1, "default"), "[REDACTED]")


if __name__ == "__main__":
    from pyspark.tests.test_conf_redact import *  # noqa: F401

    try:
        import xmlrunner  # type: ignore[import]

        testRunner = xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2)
    except ImportError:
        testRunner = None
    unittest.main(testRunner=testRunner, verbosity=2)
